(* 
 * Dusty deBoer, Kansas State University.
 *
 * Based on basicwin.sml, (C) 1990 J.H. Reppy; and goodbye.sml, (C) 1990 AT&T.
 *)

structure XDEMO : sig

    val doit  : string option * string list -> OS.Process.status
    val main  : (string * string list) -> OS.Process.status

  end = struct

    structure EXB = EXeneBase

fun init (dpyOpt,args) =
    let
    val root  = Widget.mkRoot(GetDpy.getDpy(dpyOpt))
        handle EXB.BadAddr s =>
            (TextIO.print s; RunCML.shutdown OS.Process.failure)
               
    val styleView = Styles.mkView{name = Styles.styleName["democonv"],aliases = nil}
    val view      = (styleView, (Widget.styleOf root))
    
    fun quit ()  = (Widget.delRoot root; RunCML.shutdown OS.Process.success)
    
    val quitBttn1 = Button.textBtn (root, view, 
                        [([], Attrs.attr_label, Attrs.AV_Str "Handle Quit")])
    val quitEvt1  = Button.evtOf quitBttn1
    
    val quitBttn2 = Button.textBtn (root, view, 
                        [([], Attrs.attr_label, Attrs.AV_Str "Ignore Quit")])
    val quitEvt2  = Button.evtOf quitBttn2
        
    val slowBttn = TestButton.textBtn (root, view, 
                        [([], Attrs.attr_label, Attrs.AV_Str "Slowly Quit")])
    val slowEvt  = TestButton.evtOf slowBttn
    
    val layout =
        Box.layout (root, view, []) (Box.VtCenter[
            Box.WBox(Button.widgetOf quitBttn1),
            Box.WBox(Button.widgetOf quitBttn2),
            Box.WBox(Button.widgetOf slowBttn)
        ])
    val shellArgs =
        [([], Attrs.attr_title, Attrs.AV_Str "eXene Conventions Demo"),
         ([], Attrs.attr_iconName, Attrs.AV_Str "demo-conv")]
    val shell = Shell.shell (root, view, shellArgs) (Box.widgetOf layout)
    
    val hints = Shell.mkHints{size_hints=[],wm_hints=[ICCC.HINT_Input(true)]}
    val _     = Shell.setWMHints shell hints
    val cmEvt = Shell.deletionEvent shell 
    
    fun loop():unit =
        let 
        fun handleQuit (Button.BtnUp _) = (TextIO.print " [demo-conv quitting]\n"; quit())
          | handleQuit (_) = (loop()) 
        in CML.select
           [CML.wrap(quitEvt1, handleQuit),
            (* Events from the second quit button shall be ignored:
             * CML.wrap(quitEvt2, handleQuit),*)
            CML.wrap(cmEvt, quit),
            (* events from the slow button will be handled when received. *)
            CML.wrap(slowEvt, handleQuit)
          ]
        end
    in
        Shell.init shell;
        loop()
    end

  fun doit (dpyOpt,args) = 
      (RunCML.doit (fn () => (init (dpyOpt,args)), NONE))

  fun main (prog, "-display"::(server::args)) = 
            ((TextIO.print ("display="^server)); doit(SOME server,args))
    | main (prog, args) = doit(NONE,args)

end
