(* 
 * Dusty deBoer, Kansas State University.
 *
 * Based on basicwin.sml, (C) 1990 J.H. Reppy; and goodbye.sml, (C) 1990 AT&T.
 *)

structure XDEMO : sig

    val doit  : string option * string list -> OS.Process.status
    val main  : (string * string list) -> OS.Process.status

  end = struct

    structure EXB = EXeneBase

fun init (dpyOpt,args) =
    let
    val root  = Widget.mkRoot(GetDpy.getDpy(dpyOpt))
        handle EXB.BadAddr s =>
            (TextIO.print s; RunCML.shutdown OS.Process.failure)
               
    val styleView = Styles.mkView{name = Styles.styleName["xdemo"],aliases = nil}
    val view      = (styleView, (Widget.styleOf root))
    
    fun quit ()  = (Widget.delRoot root; RunCML.shutdown OS.Process.success)
    
    val quitBttn = Button.textBtn (root, view, [([], Attrs.attr_label, Attrs.AV_Str "Handle Quit")])
    val quitEvt  = Button.evtOf quitBttn
    
    val getBttn = Button.textBtn (root, view, [([], Attrs.attr_label, Attrs.AV_Str "Get PRIMARY Selection")])
    val getEvt  = Button.evtOf getBttn
    
    val tgtBttn = Button.textBtn (root, view, [([], Attrs.attr_label, Attrs.AV_Str "Get PRIMARY Targets")])
    val tgtEvt  = Button.evtOf tgtBttn
    
    val bigBttn = Button.textBtn (root, view, [([], Attrs.attr_label, Attrs.AV_Str "Set PRIMARY to Big String")])
    val bigEvt  = Button.evtOf bigBttn
    
    fun mkBig n s = if (n<1) then s else (mkBig (n-1) (s^"0123456789ABCDEF"))
    val bigStr = mkBig 200 ""
    val _ = TextIO.print ("Test string of "^(Int.toString (String.size bigStr))^"B allocated.\n")
    
    val smplEdit1 = SimpleEdit.simpleEdit (root, view, []) ""
    val smplEdit2 = SimpleEdit.simpleEdit (root, view, []) ""
    val smplEdit3 = SimpleEdit.simpleEdit (root, view, []) ""
    val smplEdit4 = SimpleEdit.simpleEdit (root, view, []) ""
    val smplEdit5 = SimpleEdit.simpleEdit (root, view, []) "Fourscore and seven years ago..."
    
    val ff1 = FocusFrame.focusframe (root,view,[]) 
        ((SimpleEdit.widgetOf smplEdit1),(SimpleEdit.focusableOf smplEdit1))
    val ff2 = FocusFrame.focusframe (root,view,[]) 
        ((SimpleEdit.widgetOf smplEdit2),(SimpleEdit.focusableOf smplEdit2))
    val ff3 = FocusFrame.focusframe (root,view,[]) 
        ((SimpleEdit.widgetOf smplEdit3),(SimpleEdit.focusableOf smplEdit3))
    val ff4 = FocusFrame.focusframe (root,view,[]) 
        ((SimpleEdit.widgetOf smplEdit4),(SimpleEdit.focusableOf smplEdit4))
    val ff5 = FocusFrame.focusframe (root,view,[]) 
        ((SimpleEdit.widgetOf smplEdit5),(SimpleEdit.focusableOf smplEdit5))
    
    val lbl1 = Label.label (root, view, [])
    val _    = Label.setLabel lbl1 (Label.Text("Selection as STRING:    "))
    val lbl2 = Label.label (root, view, [])
    val _    = Label.setLabel lbl2 (Label.Text("Selection as TIMESTAMP: "))
    val lbl3 = Label.label (root, view, [])
    val _    = Label.setLabel lbl3 (Label.Text("Selection as LENGTH:    "))
    val lbl4 = Label.label (root, view, [])
    val _    = Label.setLabel lbl4 (Label.Text("Valid TARGETS list:     "))
    val lbl5 = Label.label (root, view, [])
    val _    = Label.setLabel lbl5 (Label.Text("Test text widget:       "))
        
    val layout =
        Box.layout (root, view, []) (Box.VtCenter[
            Box.WBox(Button.widgetOf quitBttn),
            Box.WBox(Button.widgetOf getBttn),
            Box.WBox(Button.widgetOf tgtBttn),
            Box.WBox(Button.widgetOf bigBttn),
            Box.WBox (Box.widgetOf (Box.layout (root,view,[])
             (Box.HzCenter[
                Box.WBox(Label.widgetOf lbl1),
                Box.WBox(FocusFrame.widgetOf ff1)
            ]))),
            Box.WBox (Box.widgetOf (Box.layout (root,view,[])
             (Box.HzCenter[
                Box.WBox(Label.widgetOf lbl2),
                Box.WBox(FocusFrame.widgetOf ff2)
            ]))),
            Box.WBox (Box.widgetOf (Box.layout (root,view,[])
             (Box.HzCenter[
                Box.WBox(Label.widgetOf lbl3),
                Box.WBox(FocusFrame.widgetOf ff3)
            ]))),
            Box.WBox (Box.widgetOf (Box.layout (root,view,[])
             (Box.HzCenter[
                Box.WBox(Label.widgetOf lbl4),
                Box.WBox(FocusFrame.widgetOf ff4)
            ]))),
            Box.WBox (Box.widgetOf (Box.layout (root,view,[])
             (Box.HzCenter[
                Box.WBox(Label.widgetOf lbl5),
                Box.WBox(FocusFrame.widgetOf ff5)
            ])))
        ])
    val shellArgs =
        [([], Attrs.attr_title, Attrs.AV_Str "X Resources"),
         ([], Attrs.attr_iconName, Attrs.AV_Str "xres")]
    val shell = Shell.shell (root, view, shellArgs) (Box.widgetOf layout)
    
    val id1 = Shell.addFocusableFirst shell (FocusFrame.focusableOf ff1)
    val id2 = Shell.addFocusableAfter shell (id1,FocusFrame.focusableOf ff2)
    val id3 = Shell.addFocusableAfter shell (id2,FocusFrame.focusableOf ff3)
    val id4 = Shell.addFocusableAfter shell (id3,FocusFrame.focusableOf ff4)
    val id5 = Shell.addFocusableAfter shell (id4,FocusFrame.focusableOf ff5)
    
    val hints = Shell.mkHints{size_hints=[],wm_hints=[ICCC.HINT_Input(true)]}
    val _     = Shell.setWMHints shell hints
    val cmEvt = Shell.deletionEvent shell 
    
    fun loop():unit =
        let 
        fun reqPri (tgt,time) = Shell.requestSelection shell (ICCC.Sel_PRIMARY,tgt,time)
        fun handleQuit (Button.BtnUp _)   = (TextIO.print " [xdemo quitting]\n"; quit())
          | handleQuit (_) = (loop()) 
        fun handleGet  (Button.BtnUp (b,t)) = 
                let
                fun recv ((ICCC.Val_STRING s)) =
                        SimpleEdit.setString smplEdit1 s
                  | recv ((ICCC.Val_COMPOUND_TEXT s)) =
                        SimpleEdit.setString smplEdit1 s
                  | recv ((ICCC.Val_TIMESTAMP t)) =
                        SimpleEdit.setString smplEdit2 
                            (Real.toString (EXB.XTime.toReal t))
                  | recv ((ICCC.Val_LENGTH l)) =
                        SimpleEdit.setString smplEdit3 (Int.toString l)
                  | recv _ = ()
                val _ = SimpleEdit.setString smplEdit1 ""
                val _ = SimpleEdit.setString smplEdit2 ""
                val _ = SimpleEdit.setString smplEdit3 ""
                val _ = case (reqPri ((ICCC.Tgt_MULTIPLE [ICCC.Tgt_STRING,
                            ICCC.Tgt_TIMESTAMP,ICCC.Tgt_LENGTH]), t)) of
                        (SOME (ICCC.Val_MULTIPLE (mv))) => List.app recv mv 
                       | _ => (case (reqPri (ICCC.Tgt_STRING,t)) of
                                 SOME (ICCC.Val_STRING s) => 
                                    (SimpleEdit.setString smplEdit1 s)
                               | SOME (ICCC.Val_COMPOUND_TEXT s) => 
                                    (SimpleEdit.setString smplEdit1 s)
                               | _ => ())
                in loop() end 
          | handleGet  (_) = (loop())
        fun handleTgt (Button.BtnUp (b,t)) =
                (case (reqPri (ICCC.Tgt_TARGETS,t)) of
                    SOME (ICCC.Val_TARGETS ts) => 
                        (SimpleEdit.setString smplEdit4 
                            (List.foldr 
                                (fn (t,s) => ((ICCC.tgtToString t)^","^s)) 
                                 "" ts);
                         loop())
                  | _ => (loop()) )
          | handleTgt  (_) = (loop())
        fun handleBig (Button.BtnUp (b,t)) =
                ((case (Shell.acquireSelection shell 
                    (ICCC.Sel_PRIMARY,t,(ICCC.convertString bigStr))) of
                 SOME _ => TextIO.print "Set selection to big string.\n"
               | NONE => TextIO.print "Failed to set selection to big string.\n");
                 loop())
          | handleBig (_) = (loop())
        in CML.select
           [CML.wrap(quitEvt, handleQuit),
            CML.wrap(getEvt, handleGet),
            CML.wrap(tgtEvt, handleTgt),
            CML.wrap(bigEvt, handleBig),
            CML.wrap(cmEvt, quit)
          ]
        end
    in
        Shell.init shell;
        loop()
    end

  fun doit (dpyOpt,args) = 
      (RunCML.doit (fn () => (init (dpyOpt,args)), NONE))

  fun main (prog, "-display"::(server::args)) = 
            ((TextIO.print ("display="^server)); doit(SOME server,args))
    | main (prog, args) = doit(NONE,args)

end
